<cfcomponent output="false"
			 hint="This component exposes methods for manipulating in-database trees
			 based on the nested set model.  All you need to do is ensure your table
			 has a numeric primary key (the 'id' field), numeric left/right fields (the
			 'lpos' and 'rpos' fields, and a sortable field for ordering (the 'sort'
			 field), if you want to use the sortChildren and/or sortSiblings methods.
			 The names of the actual fields (along with the datasource and table name)
			 are passed as arguments to the init method, and all are optional.
			 
			 While the nested set model theoretically allows for multiple 'root' nodes,
			 this implementation doesn't support that.  There must be a single root
			 node that all other nodes are descendants of.  While parts of the code will
			 work with multiple top-level nodes, many methods will fail, and none are
			 indended to be used with a multi-root tree.
			 
			 It must be mentioned that there is no guarentee that the left/right
			 positions maintained by this object will remain fully compressed.  It is
			 entirely possible that spaces will develop.
			 ">

	<cffunction name="init" access="public" output="false" returntype="treemanager"
				hint="I initialize this treemanager object, and return myself.">
		<cfargument name="dsn" type="string" required="true"
					hint="The datasource to access the database though." />
		<cfargument name="table" type="string" required="true"
					hint="The database table that the tree objects are stored in." />
		<cfargument name="sortField" type="string" required="false" default="name"
					hint="The field that should control sorting operations.  This
					field needn't be unique, it will always be backed up with the
					'id' field to ensure a stable result." />
		<cfargument name="lposField" type="string" required="false" default="lpos"
					hint="The field that stores the left number of nodes." />
		<cfargument name="rposField" type="string" required="false" default="rpos"
					hint="The field that stores the right number of nodes." />
		<cfargument name="idField" type="string" required="false" default="id"
					hint="The field that stores the unique id of a node." />
		<cfset variables.my.dsn = dsn />
		<cfset variables.table = table />
		<cfset variables.name = sortField />
		<cfset variables.lpos = lposField />
		<cfset variables.rpos = rposField />
		<cfset variables.id = idField />
		
		<cfreturn this />
	</cffunction>



	<cffunction name="getChildIds" access="public" output="false" returntype="string"
				hint="I return a list of IDs for the child nodes of the specified node.">
		<cfargument name="id" type="numeric" required="true"
					hint="The node to find the children of." />
		<cfset var getChildIds = "" />
		<cfquery datasource="#variables.my.dsn#" name="getChildIds">
			SELECT child.#variables.id# AS id
			FROM #table# parent, #table# child
			WHERE parent.#variables.id# = <cfqueryparam cfsqltype="cf_sql_integer" value="#id#" />
				AND child.#lpos# BETWEEN parent.#lpos# AND parent.#rpos#
				AND child.#variables.id# <> parent.#variables.id#
				AND NOT EXISTS (
					SELECT *
					FROM #table# innerLevel
					WHERE innerLevel.#lpos# BETWEEN parent.#lpos# AND parent.#rpos#
						AND child.#lpos# BETWEEN innerLevel.#lpos# AND innerLevel.#rpos#
						AND #variables.id# NOT IN (child.#variables.id#)
						AND #variables.id# <> parent.#variables.id#
				)
			ORDER BY child.#lpos#
		</cfquery>
		<cfreturn valueList(getChildIds.id) />
	</cffunction>
	
	
	
	<cffunction name="getFirstChild" access="public" output="false" returntype="struct"
				hint="I return a struct with id, lpos, and rpos of the specified node's
				first child."
				throws="IllegalStateException.NoChildrenException">
		<cfargument name="id" type="numeric" required="true"
					hint="The id of the node to find the first child of." />
		<cfset var get = "" />
		<cfset var result = structNew() />
		<cfquery datasource="#variables.my.dsn#" name="get">
			SELECT #variables.id# AS id, #rpos# AS rpos, #lpos# AS lpos
			FROM #table#
			WHERE #lpos# > (
					SELECT #lpos#
					FROM #table#
					WHERE #variables.id# = <cfqueryparam cfsqltype="cf_sql_integer" value="#id#" />
				)
				AND #rpos# < (
					SELECT #rpos#
					FROM #table#
					WHERE #variables.id# = <cfqueryparam cfsqltype="cf_sql_integer" value="#id#" />
				)
			ORDER BY #lpos#
			LIMIT 1
		</cfquery>
		<cfif NOT get.recordCount EQ 1>
			<cfthrow type="IllegalStateException.NoChildrenException"
				message="The node you specified doesn't have any child nodes" />
		</cfif>
		<cfset result.id = get.id />
		<cfset result.lpos = get.lpos />
		<cfset result.rpos = get.rpos />
		<cfreturn result />
	</cffunction>
	
	
	
	<cffunction name="getNextSibling" access="public" output="false" returntype="struct"
				hint="Returns a struct with the id, lpos, and rpos of the specified
				node's next sibling."
				throws="IllegalStateException.NoNextSiblingException">
		<cfargument name="id" type="numeric" required="true"
					hint="The id of the node to find the next sibling of." />
		<cfset var get = "" />
		<cfset var parent = "" />
		<cfset var result = structNew() />
		<cftry>
			<cfset parent = getParent(id) />
			<cfcatch type="IllegalStateException.NoParentException">
				<!--- whatever --->
			</cfcatch>
		</cftry>
		<cfquery datasource="#variables.my.dsn#" name="get">
			SELECT #variables.id# AS id, #rpos# AS rpos, #lpos# AS lpos
			FROM #table#
			WHERE #lpos# > (
					SELECT #rpos#
					FROM #table#
					WHERE #variables.id# = <cfqueryparam cfsqltype="cf_sql_integer" value="#id#" />
				)
				<cfif isStruct(parent)>
					AND #lpos# < <cfqueryparam cfsqltype="cf_sql_integer" value="#parent.rpos#" />
				</cfif>
			ORDER BY #lpos#
			LIMIT 1
		</cfquery>
		<cfif NOT get.recordCount EQ 1>
			<cfthrow type="IllegalStateException.NoNextSiblingException"
				message="The node you specified doesn't have a next sibling node" />
		</cfif>
		<cfset result.id = get.id />
		<cfset result.lpos = get.lpos />
		<cfset result.rpos = get.rpos />
		<cfreturn result />
	</cffunction>
	
	
	
	<cffunction name="getNode" access="public" output="false" returntype="struct"
				hint="Returns a struct with the id, lpos, and rpos of the specified
				node"
				throws="IllegalStateException.NoSuchNodeException">
		<cfargument name="id" type="numeric" required="true"
					hint="The id of the node to get info about." />
		<cfset var get = "" />
		<cfset var result = structNew() />
		<cfquery datasource="#variables.my.dsn#" name="get">
			SELECT #variables.id# AS id, #rpos# AS rpos, #lpos# AS lpos
			FROM #table#
			WHERE #variables.id# = <cfqueryparam cfsqltype="cf_sql_integer" value="#id#" />
		</cfquery>
		<cfif NOT get.recordCount EQ 1>
			<cfthrow type="IllegalStateException.NoSuchNodeException"
				message="The node you specified doesn't exit" />
		</cfif>
		<cfset result.id = get.id />
		<cfset result.lpos = get.lpos />
		<cfset result.rpos = get.rpos />
		<cfreturn result />
	</cffunction>

	

	<cffunction name="getNodeList" access="public" output="false" returntype="query"
				hint="I return a recordset containing the full list of nodes, using
				the specified columnList, with the addition of a 'depth' column, which
				contains the depth of each node in the tree.">
		<cfargument name="columnList" type="string" required="false" default="*"
					hint="The list of columns to select for the recordset.  This value
					will be injected directly into the SQL, so it must be sanitized
					before passing.  Since the string is injected into the SELECT
					clause of the statement, you cannot perform joins, but you can
					use subqueries.  The statement you're injecting into is a single
					table SELECT on the unaliased table name, and ordered by the
					lpos column." />
		<cfset var getNodeList = "" />
		<cfquery datasource="#variables.my.dsn#" name="getNodeList">
			SELECT #preserveSingleQuotes(columnList)#,
				(
					SELECT COUNT(t2.#variables.id#)
					FROM #table# t2
					WHERE #table#.#lpos# BETWEEN t2.#lpos# AND t2.#rpos#
				) AS depth
			FROM #table#
			ORDER BY #lpos#
		</cfquery>
		<cfreturn getNodeList />
	</cffunction>



	<cffunction name="getParent" access="package" output="false" returntype="struct"
				hint="Returns a struct with the id, lpos, and rpos of the specified
				node's parent."
				throws="IllegalStateException.NoParentException">
		<cfargument name="id" type="numeric" required="true"
					hint="The id of the node to find the parent of." />
		<cfset var get = "" />
		<cfset var result = structNew() />
		<cfquery datasource="#variables.my.dsn#" name="get">
			SELECT #variables.id# AS id, #lpos# AS lpos, #rpos# AS rpos
			FROM #table#
			WHERE #lpos# < (
					SELECT #lpos#
					FROM #table#
					WHERE #variables.id# = <cfqueryparam cfsqltype="cf_sql_integer" value="#id#" />
				)
				AND #rpos# > (
					SELECT #rpos#
					FROM #table#
					WHERE #variables.id# = <cfqueryparam cfsqltype="cf_sql_integer" value="#id#" />
				)
			ORDER BY #lpos# DESC
			LIMIT 1
		</cfquery>
		<cfif NOT get.recordCount EQ 1>
			<cfthrow type="IllegalStateException.NoParentException"
				message="The node you specified doesn't have a parent node" />
		</cfif>
		<cfset result.id = get.id />
		<cfset result.lpos = get.lpos />
		<cfset result.rpos = get.rpos />
		<cfreturn result />
	</cffunction>
	
	
	
	<cffunction name="getPreviousSibling" access="public" output="false" returntype="struct"
				hint="Returns a struct with the id, lpos, and rpos of the specified
				node's previous sibling."
				throws="IllegalStateException.NoPreviousSiblingException">
		<cfargument name="id" type="numeric" required="true"
					hint="The id of the node to find the next previous of." />
		<cfset var get = "" />
		<cfset var parent = "" />
		<cfset var result = structNew() />
		<cftry>
			<cfset parent = getParent(id) />
			<cfcatch type="IllegalStateException.NoParentException">
				<!--- whatever --->
			</cfcatch>
		</cftry>
		<cfquery datasource="#variables.my.dsn#" name="get">
			SELECT #variables.id# AS id, #rpos# AS rpos, #lpos# AS lpos
			FROM #table#
			WHERE #rpos# < (
					SELECT #lpos#
					FROM #table#
					WHERE #variables.id# = <cfqueryparam cfsqltype="cf_sql_integer" value="#id#" />
				)
				<cfif isStruct(parent)>
					AND #lpos# > <cfqueryparam cfsqltype="cf_sql_integer" value="#parent.lpos#" />
				</cfif>
			ORDER BY #rpos# DESC
			LIMIT 1
		</cfquery>
		<cfif NOT get.recordCount EQ 1>
			<cfthrow type="IllegalStateException.NoPreviousSiblingException"
				message="The node you specified doesn't have a previous sibling node" />
		</cfif>
		<cfset result.id = get.id />
		<cfset result.lpos = get.lpos />
		<cfset result.rpos = get.rpos />
		<cfreturn result />
	</cffunction>



	<cffunction name="initializeHierarchy" access="public" output="false" returntype="void"
				hint="I initialize the tree hierarchy with the specified node as the root.">
		<cfargument name="id" type="numeric" required="true"
					hint="The id of the node to start as the root of the hierarchy." />
		<cfset var get = "" />
		<cfquery datasource="#variables.my.dsn#" name="get">
			SELECT MAX(#lpos#) AS lpos, MAX(#rpos#) AS rpos
			FROM #table#
		</cfquery>
		<cfif isNumeric(get.lpos) OR isNumeric(get.rpos)>
			<cfthrow type="IllegalStateException"
				message="You cannot initialize the object hierarchy because it already exists" />
		</cfif>
		<cfquery datasource="#variables.my.dsn#" name="">
			UPDATE #table# SET
				#lpos# = 1,
				#rpos# = 2
			WHERE #variables.id# = <cfqueryparam cfsqltype="cf_sql_integer" value="#id#" />
		</cfquery>
	</cffunction>



	<cffunction name="injectIntoHierarchy" access="public" output="false" returntype="void"
				hint="I inject a node into the hierarchy.  In terms of the database, I take
				a record that doesn't currently belong to the tree and add it into the tree,
				using the specified node as it's parent.  Newly added nodes will always end
				up as the last child of the parent.">
		<cfargument name="id" type="numeric" required="true"
					hint="The id of the record to inject." />
		<cfargument name="parentID" type="numeric" required="true"
					hint="The id of the node the new node should be a child of." />
		<cfset var get = "" />
		<cfquery datasource="#variables.my.dsn#" name="get">
			SELECT #rpos# AS rpos
			FROM #table#
			WHERE #variables.id# = <cfqueryparam cfsqltype="cf_sql_integer" value="#parentID#" />
		</cfquery>
		<cfset shift(get.rpos, 2) />
		<cfquery datasource="#variables.my.dsn#" name="">
			UPDATE #table# SET
				#lpos# = <cfqueryparam cfsqltype="cf_sql_integer" value="#get.rpos#" />,
				#rpos# = <cfqueryparam cfsqltype="cf_sql_integer" value="#get.rpos + 1#" />
			WHERE #variables.id# = <cfqueryparam cfsqltype="cf_sql_integer" value="#id#" />
		</cfquery>
	</cffunction>



	<cffunction name="makeChildOfNode" access="public" output="false" returntype="void"
				hint="I nest an node under another node.  If the new parent node is a
				descendant of the node being moved, an exception is raised."
				throws="IllegalStateException.InvalidNestingRequest">
		<cfargument name="id" type="numeric" required="true"
					hint="The node to nest." />
		<cfargument name="parentId" type="numeric" required="true"
					hint="The node's new parent node." />
		<cfset var node = getNode(id) />
		<cfset var parent = getNode(parentId) />
		<cfif parent.lpos GTE node.lpos AND parent.rpos LTE node.rpos>
			<cfthrow type="IllegalStateException.InvalidNestingRequest"
				message="You can't nest a node under one of it's decendants." />
		</cfif>
		<cftry>
			<cfset moveSubtree(id, parent.rpos) />
			<cfcatch type="IllegalStateException.NoSuchNodeException">
				<!--- whatever --->
			</cfcatch>
		</cftry>
	</cffunction>


	
	<cffunction name="moveDown" access="public" output="false" returntype="void"
				hint="I move a node down one slot.  In other words, swap it with it's next
				sibling.  If the node doesn't have a next sibling, nothing happens.">
		<cfargument name="id" type="numeric" required="true"
					hint="The id of the node to move down." />
		<cftry>
			<cfset moveSubtree(id, getNextSibling(id).rpos + 1) />
			<cfcatch type="IllegalStateException.NoNextSiblingException">
				<!--- whatever --->
			</cfcatch>
		</cftry>
	</cffunction>



	<cffunction name="moveToBottom" access="public" output="false" returntype="void"
				hint="I move a node to the bottom of the sibling list.  If the node doesn't
				have siblings or is already at the bottom, nothing happens.">
		<cfargument name="id" type="numeric" required="true"
					hint="The id of the node to move to the bottom." />
		<cfset moveSubtree(id, getParent(id).rpos) />
	</cffunction>

	

	<cffunction name="moveToTop" access="public" output="false" returntype="void"
				hint="I move a node to the top of the sibling list.  If the node doesn't
				have siblings or is already at the top, nothing happens.">
		<cfargument name="id" type="numeric" required="true"
					hint="The id of the node to move to the top." />
		<cfset moveSubtree(id, getParent(id).lpos + 1) />
	</cffunction>



	<cffunction name="moveUp" access="public" output="false" returntype="void"
				hint="I move a node up one slot.  In other words, swap it with it's previous
				sibling.  If the node doesn't have a previous sibling, nothing happens.">
		<cfargument name="id" type="numeric" required="true"
					hint="The id of the node to move up." />
		<cftry>
			<cfset moveSubtree(id, getPreviousSibling(id).lpos) />
			<cfcatch type="IllegalStateException.NoPreviousSiblingException">
				<!--- whatever --->
			</cfcatch>
		</cftry>
	</cffunction>



	<cffunction name="nestObject" access="public" output="false" returntype="void"
				hint="I nest an object under it's previous sibling.  If the node doesn't
				have a previous sibling, nothing happens.">
		<cfargument name="id" type="numeric" required="true"
					hint="The node to nest." />
		<cftry>
			<cfset moveSubtree(id, getPreviousSibling(id).rpos) />
			<cfcatch type="IllegalStateException.NoPreviousSiblingException">
				<!--- whatever --->
			</cfcatch>
		</cftry>
	</cffunction>

	

	<cffunction name="removeFromHierarchy" access="public" output="false" returntype="void"
				hint="I remove the specified node from the tree hierarchy.  Note that the
				record will continue to exist in the database, it just won't be linked
				into the hierarchy">
		<cfargument name="id" type="numeric" required="true"
					hint="The id of the node to remove." />
		<cfset var get = "" />
		<! --- get the node to delete and remove it from the hierarchy --- >
		<cfquery datasource="#variables.my.dsn#" name="get">
			SELECT #lpos# AS lpos, #rpos# AS rpos
			FROM #table#
			WHERE #variables.id# = <cfqueryparam cfsqltype="cf_sql_integer" value="#id#" />
		</cfquery>
		<cfif isNumeric(get.lpos) OR isNumeric(get.rpos)>
			<cfquery datasource="#variables.my.dsn#" name="">
				UPDATE #table# SET
					#lpos# = null,
					#rpos# = null
				WHERE #variables.id# = <cfqueryparam cfsqltype="cf_sql_integer" value="#id#" />
			</cfquery>
			<cfset shiftRange(get.lpos, get.rpos, -1) />
			<cfset shift(get.rpos, -2) />
		</cfif>
	</cffunction>

	

	<cffunction name="sortChildren" access="public" output="false" returntype="void"
				hint="I sort the children of the specified node according to the configured
				sort field, using the id field as backup to ensure a stable sort.  The sort
				method used will be most performant when the nodes are already close to
				sorted (under the assumption that if you want the nodes sorted, you'll resort
				them after every operation that might have disturbed their ordering, and so
				rarely have more than a node or two to reorder).">
		<cfargument name="parentID" type="numeric" required="true"
					hint="The id of the node whose children need sorting" />
		<cfargument name="descending" type="boolean" required="false" default="false"
					hint="Whether the sorting should be descending (reversed) order" />
		<!---
			TODO: fix the sort to be more efficient

			This is a horribly inefficient sorting algorithm, but it works and was
			simple to implement.  It should perform reasonably well when the list is
			already mostly sorted, but if it's all scrambled, it'll be horrendously
			slow.
		--->
		<cfset var getDesiredOrder = "" />
		<cfset var getCurrentItem = "" />
		<cfset var i = "" />
		<cfset var childIds = getChildIds(parentID) />
		<cfif listLen(childIds) LTE 1>
			<!--- can't sort children if there aren't any, and no need to sort if there's only one --->
			<cfreturn />
		</cfif>
		<cfquery datasource="#variables.my.dsn#" name="getDesiredOrder">
			SELECT #variables.id# AS id, #name#
			FROM #table#
			WHERE #variables.id# IN (<cfqueryparam cfsqltype="cf_sql_integer" value="#childIds#" list="true" />)
			ORDER BY #name#<cfif descending> DESC</cfif>, #variables.id#<cfif descending> DESC</cfif>
		</cfquery>
		<cfloop from="1" to="#getDesiredOrder.recordCount#" index="i">
			<cfloop condition="true">
				<cfquery datasource="#variables.my.dsn#" name="getCurrentItem">
					SELECT #variables.id# AS id
					FROM #table#
					WHERE #variables.id# IN (<cfqueryparam cfsqltype="cf_sql_integer" value="#getChildIds(parentID)#" list="true" />)
					ORDER BY #lpos#
					LIMIT #i - 1#, 1
				</cfquery>
				<cfif getCurrentItem.id EQ getDesiredOrder.id[i]>
					<cfbreak />
				</cfif>
				<cfset moveToBottom(getCurrentItem.id) />
			</cfloop>
		</cfloop>
	</cffunction>

	

	<cffunction name="sortSiblings" access="public" output="false" returntype="void"
				hint="I sort the node and it's siblings according to the configured
				sort field, using the id field as backup to ensure a stable sort.
				This method is currently implemented as nothing more than a call
				to sortChildren passing the id of the specified node's parent.  That
				means that you can't sort the root level of the hierarchy, which should
				be expected, since a tree never has more than one node at it's root.">
		<cfargument name="id" type="numeric" required="true"
					hint="The id of one of the sibling nodes that need sorting" />
		<cfargument name="descending" type="boolean" required="false" default="false"
					hint="Whether the sorting should be descending (reversed) order" />
		<cfset sortChildren(getParent(id).id, descending) />
	</cffunction>



	<cffunction name="unnestObject" access="public" output="false" returntype="void"
				hint="I unnest an object (make it a sibling of it's parent).  If the node
				doesn't have a parent, nothing happens.">
		<cfargument name="id" type="numeric" required="true"
					hint="The node to unnest." />
		<cftry>
			<cfset moveSubtree(id, getParent(id).rpos + 1) />
			<cfcatch type="IllegalStateException.NoParentException">
				<!--- whatever --->
			</cfcatch>
		</cftry>
	</cffunction>



<!----------------------------------------------------------------------
-- Private helper methods
----------------------------------------------------------------------->
	<cffunction name="moveSubtree" access="private" output="false" returntype="void"
				hint="Thid moves the specified node to a new place in the tree.">
		<cfargument name="id" type="numeric" required="true"
					hint="The node to move." />
		<cfargument name="destPos" type="numeric" required="true"
					hint="The new lpos for the node.  Note that this is not necessarily
					the lpos the node will keep, because after the move the nodes may
					be recompressed and the lpos shifted." />
		<cfset var treeSize = "" />
		<cfset var get = "" />
		<cfset var lpos = "" />
		<cfset var rpos = "" />
		<cfquery datasource="#variables.my.dsn#" name="get">
			SELECT #variables.lpos# AS lpos, #variables.rpos# AS rpos
			FROM #table#
			WHERE #variables.id# = <cfqueryparam cfsqltype="cf_sql_integer" value="#id#" />
		</cfquery>
		<cfset lpos = get.lpos />
		<cfset rpos = get.rpos />
		<cfset treeSize = rpos - lpos + 1 />
		<cfset shift(destPos, treeSize) />
		<cfif lpos GTE destPos>
			<cfset lpos = lpos + treeSize />
			<cfset rpos = rpos + treeSize />
		</cfif>
		<cfset shiftRange(lpos, rpos, destPos - lpos) />
		<cfset shift(rpos + 1, -1 * treeSize) />
	</cffunction>



	<cffunction name="shift" access="private" output="false" returntype="void"
				hint="I move all positions greater than a certain position a certain
				distance.  This is typically used to make or close hole in the
				hierarchy when moving subtrees around.">
		<cfargument name="startPos" type="numeric" required="true"
					hint="The lowest position to move." />
		<cfargument name="delta" type="numeric" required="true"
					hint="The distance to move (may be negative)." />
		<cfquery datasource="#variables.my.dsn#" name="">
			UPDATE #table# SET
				#lpos# = #lpos# + <cfqueryparam cfsqltype="cf_sql_integer" value="#delta#" />
			WHERE #lpos# >= <cfqueryparam cfsqltype="cf_sql_integer" value="#startPos#" />
		</cfquery>
		<cfquery datasource="#variables.my.dsn#" name="">
			UPDATE #table# SET
				#rpos# = #rpos# + <cfqueryparam cfsqltype="cf_sql_integer" value="#delta#" />
			WHERE #rpos# >= <cfqueryparam cfsqltype="cf_sql_integer" value="#startPos#" />
		</cfquery>
	</cffunction>



	<cffunction name="shiftRange" access="private" output="false" returntype="void"
				hint="I move a range of positions a certain distance.  Usually used for
				moving a subtree into/out of a hole in the hierarchy created with the
				shift method.">
		<cfargument name="startPos" type="numeric" required="true"
					hint="The starting position of the range." />
		<cfargument name="endPos" type="numeric" required="true"
					hint="The ending position of the range." />
		<cfargument name="delta" type="numeric" required="true"
					hint="The distance to move." />
		<cfquery datasource="#variables.my.dsn#" name="">
			UPDATE #table# SET
				#lpos# = #lpos# + <cfqueryparam cfsqltype="cf_sql_integer" value="#delta#" />
			WHERE #lpos# BETWEEN <cfqueryparam cfsqltype="cf_sql_integer" value="#startPos#" />
				AND <cfqueryparam cfsqltype="cf_sql_integer" value="#endPos#" />
		</cfquery>
		<cfquery datasource="#variables.my.dsn#" name="">
			UPDATE #table# SET
				#rpos# = #rpos# + <cfqueryparam cfsqltype="cf_sql_integer" value="#delta#" />
			WHERE #rpos# BETWEEN <cfqueryparam cfsqltype="cf_sql_integer" value="#startPos#" />
				AND <cfqueryparam cfsqltype="cf_sql_integer" value="#endPos#" />
		</cfquery>
	</cffunction>

</cfcomponent>